/* ==================================================================
   File: Commands.h

   This file provides functions that send commands to, and receive
   responses from, the flight simulation.

   void SayHello(HWND)
       Takes a parent window handle for display, sends the "hello"
       command to the server, and then displays the returned time
       and date in a message box.

    void SetPosition(double x, double y, double Alt, double Psi)
        Takes x- and y-position and altitude in feet, and aircraft
        bearing in degrees, and sets the trim command variables.
        --- CALL THIS FUNCTION BEFORE TRIMMING ---

    bool TrimAlpha(double alpha)
        Takes angle of attack in degrees, sends the trim
        command to the simulation, and receives the trim response.
        Returns TRUE on success, FALSE on failure.

    bool TrimSpeed(double speed)
        Takes aircraft speed in knots, sends the trim command to
        the simulation, and receives the trim response.
        Returns TRUE on success, FALSE on failure.

    bool TrimDele(double dele)
        Takes the elevator angle in degrees, sends the trim command
        to the simulation and receives the trim response.
        Returns TRUE on success, FALSE on failure.

    void DisplayTrim(HWND)
        Takes a parent window handle for display, then displays the
        trim values in a message box.

    bool AdvanceSimulation(double deltaT)
        Takes the time to advance the simulation into the future,
        in seconds, sends the advance command to the simulation, then
        recieves the response.  Returns TRUE on success, FALSE
        on failure.

            Copyright (C) 2014 Richard C. Wagner
   This file is offered under the GNU General Public License.
   Please see the license statement at the end of the file.
   =============================================================== */

void SayHello(HWND hWND)
{
    char OutStr[100];

    *hc_command_value = hello_cmd_value;
    SendCommand(hello_command, sizeof(hello_command));
    ReceiveResponse(hello_response, sizeof(hello_response));
    sprintf(OutStr, "Time:  %02d:%02d:%02d   Date:  %d/%d/%d",
            *hr_hours, *hr_minutes, *hr_seconds,
            *hr_month, *hr_day, *hr_year);
    MessageBox(hWND, OutStr, "Simulation Hello Response", MB_OK);
    return;
}


// =======================  Trim Commands  ===========================

void DisplayTrim(HWND hWND)
{
    char OutStr[1000],TempStr[100];

    if (*tr_trim_result == 0)
        strcpy(OutStr, ">>> FAILED <<<\n");
    else
        strcpy(OutStr, "--- Success ---\n");
    sprintf(TempStr, "Alpha = %f deg.\n", RadiansToDegrees(*tr_alpha));
    strcat(OutStr, TempStr);
    sprintf(TempStr, "Beta = %f deg.\n", RadiansToDegrees(*tr_beta));
    strcat(OutStr, TempStr);
    sprintf(TempStr, "Speed = %f kts.\n", FtPerSecToKts(*tr_speed));
    strcat(OutStr, TempStr);
    sprintf(TempStr, "Delta-A = %f deg.\n", RadiansToDegrees(*tr_dela));
    strcat(OutStr, TempStr);
    sprintf(TempStr, "Delta-E = %f deg.\n", RadiansToDegrees(*tr_dele));
    strcat(OutStr, TempStr);
    sprintf(TempStr, "Delta-R = %f deg.\n", RadiansToDegrees(*tr_delr));
    strcat(OutStr, TempStr);
    sprintf(TempStr, "Thrust = %f lb.\n", *tr_thrust);
    strcat(OutStr, TempStr);
    sprintf(TempStr, "Throttle = %f %%\n", *tr_throttle * 100);
    strcat(OutStr, TempStr);
    MessageBox(hWND, OutStr, "Trim Results", MB_OK);
    return;
}

void SetPosition(double x, double y, double alt, double psi)
{
    *tc_x = x;
    *tc_y = y;
    *tc_altitude = -alt;
    *tc_psi = DegreesToRadians(psi);
    return;
}

void SetAlpha(double alpha)
{
    *tc_trim_strategy = by_alpha;
    *tc_alpha = alpha;
    return;
}

void SetSpeed(double speed)
{
    *tc_trim_strategy = by_speed;
    *tc_speed = speed;
    return;
}

void SetDelE(double dele)
{
    *tc_trim_strategy = by_dele;
    *tc_dele = dele;
    return;
}

void CopyDeltas()
{
    *ac_dela     = *tr_dela;
    *ac_dele     = *tr_dele;
    *ac_delr     = *tr_delr;
    *ac_throttle = *tr_throttle;
    return;
}

bool TrimAircraft()
{
    bool sresult,rresult;

    *tc_command_value = trim_cmd_value;
    sresult = SendCommand(trim_command, sizeof(trim_command));
    rresult = ReceiveResponse(trim_response, sizeof(trim_response));
    return sresult && rresult;
}

bool TrimAlpha(double alpha)
{
    bool result;

    SetAlpha(alpha);
    result = TrimAircraft();
    CopyDeltas();
    return result;
}

bool TrimSpeed(double speed)
{
    bool result;

    SetSpeed(speed);
    result = TrimAircraft();
    CopyDeltas();
    return result;
}

bool TrimDele(double dele)
{
    bool result;

    SetDelE(dele);
    result = TrimAircraft();
    CopyDeltas();
    return result;
}


// ======================  Advance Command  =========================

bool AdvanceSimulation(double deltaT)
{
    bool sresult, rresult;

    *ac_command_value = advance_cmd_value;
    *ac_advance_time = deltaT;
    sresult = SendCommand(advance_command, sizeof(advance_command));
    rresult = ReceiveResponse(advance_response, sizeof(advance_response));
    return sresult && rresult;
}


/* ====================  License Statement  =========================
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
   =============================================================== */
