/* ==================================================================
   File: DataValues.h

   This file creates the data values needed for the client to
   communicate with the simulation.

   Notionally, the easiest way to create the messages to be sent to
   and received from the simulation is to use structures, with the
   individual fields specified in order.  In C, however, there is
   no guarantee that the structure, when created in memory, will
   be contiguous.  The gcc compiler can insert a pad into the
   memory image to comply with memory alignment issues.  The
   simulation requires the message fields to be contiguous.  And so
   we revert to another technique.  We create character arrays of
   the correct size, then assign type-specific pointers to addresses
   within the arrays.  Arrays in C are guaranteed to be contiguous
   in memory, and the pointers will ensure that data is stored at
   the correct location, in the correct format.

   --- Note: int = 4 bytes, double = 8 bytes

   --- Remember that the fields are pointers, don't forget ---
   ---    to use the * when referencing their values!      ---
   ---  e.g.  *ac_command_value = advance_command_value;   ---

            Copyright (C) 2014 Richard C. Wagner
   This file is offered under the GNU General Public License.
   Please see the license statement at the end of the file.
   =============================================================== */

// ======================  Command Values ===========================

#define hello_cmd_value   0
#define advance_cmd_value 1
#define trim_cmd_value    2


// ====================  Trim Strategy Values  ======================

#define by_alpha 0
#define by_speed 1
#define by_dele  2


// ============  Hello Command and Response Messages  ===============

// Command message
char hello_command[4];

// Message field
int *hc_command_value = (int *)&hello_command[0];  // Base address
                      // Last address: 3   Total size: 4 bytes


// Response message
char hello_response[28];

// Response fields
int *hr_command_value = (int *)&hello_response[0],   // Base address
    *hr_seconds       = (int *)&hello_response[4],   // + 4 bytes
    *hr_minutes       = (int *)&hello_response[8],   // + 4 bytes
    *hr_hours         = (int *)&hello_response[12],  // + 4 bytes
    *hr_day           = (int *)&hello_response[16],  // + 4 bytes
    *hr_month         = (int *)&hello_response[20],  // + 4 bytes
    *hr_year          = (int *)&hello_response[24];  // + 4 bytes
                       // Last address: 27   Total size: 28 bytes


// ===========  Advance Command and Response Messages  ==============

// Command message
char advance_command[44];

// Command fields
int    *ac_command_value = (int *)   &advance_command[0];  // Base address
double *ac_advance_time  = (double *)&advance_command[4],  // + 4 bytes
       *ac_dela          = (double *)&advance_command[12], // + 8 bytes
       *ac_dele          = (double *)&advance_command[20], // + 8 bytes
       *ac_delr          = (double *)&advance_command[28], // + 8 bytes
       *ac_throttle      = (double *)&advance_command[36]; // + 8 bytes
                           // Last address: 51   Total size: 44 bytes


// Response message
char advance_response[276];

// Response fields
int    *ar_command_value = (int *)   &advance_response[0];   // Base address
double *ar_simtime       = (double *)&advance_response[4],   // + 4 bytes
       *ar_u             = (double *)&advance_response[12],  // + 8 bytes
       *ar_v             = (double *)&advance_response[20],  // + 8 bytes
       *ar_w             = (double *)&advance_response[28],  // + 8 bytes
       *ar_p             = (double *)&advance_response[36],  // + 8 bytes
       *ar_q             = (double *)&advance_response[44],  // + 8 bytes
       *ar_r             = (double *)&advance_response[52],  // + 8 bytes
       *ar_psi           = (double *)&advance_response[60],  // + 8 bytes
       *ar_theta         = (double *)&advance_response[68],  // + 8 bytes
       *ar_phi           = (double *)&advance_response[76],  // + 8 bytes
       *ar_x             = (double *)&advance_response[84],  // + 8 bytes
       *ar_y             = (double *)&advance_response[92],  // + 8 bytes
       *ar_altitude      = (double *)&advance_response[100], // + 8 bytes
       *ar_alpha         = (double *)&advance_response[108], // + 8 bytes
       *ar_beta          = (double *)&advance_response[116], // + 8 bytes
       *ar_gamma         = (double *)&advance_response[124], // + 8 bytes
       *ar_du_dt         = (double *)&advance_response[132], // + 8 bytes
       *ar_dv_dt         = (double *)&advance_response[140], // + 8 bytes
       *ar_dw_dt         = (double *)&advance_response[148], // + 8 bytes
       *ar_dp_dt         = (double *)&advance_response[156], // + 8 bytes
       *ar_dq_dt         = (double *)&advance_response[164], // + 8 bytes
       *ar_dr_dt         = (double *)&advance_response[172], // + 8 bytes
       *ar_dpsi_dt       = (double *)&advance_response[180], // + 8 bytes
       *ar_dtheta_dt     = (double *)&advance_response[188], // + 8 bytes
       *ar_dphi_dt       = (double *)&advance_response[196], // + 8 bytes
       *ar_dx_dt         = (double *)&advance_response[204], // + 8 bytes
       *ar_dy_dt         = (double *)&advance_response[212], // + 8 bytes
       *ar_dz_dt         = (double *)&advance_response[220], // + 8 bytes
       *ar_dela          = (double *)&advance_response[228], // + 8 bytes
       *ar_dele          = (double *)&advance_response[236], // + 8 bytes
       *ar_delr          = (double *)&advance_response[244], // + 8 bytes
       *ar_thrust        = (double *)&advance_response[252], // + 8 bytes
       *ar_power         = (double *)&advance_response[260], // + 8 bytes
       *ar_throttle      = (double *)&advance_response[268]; // + 8 bytes
                            // Last address: 275   Total size: 276 bytes


// =============  Trim Command and Response Messages  ===============

// Command message
char trim_command[64];

// Command fields
int    *tc_command_value = (int *)   &trim_command[0],  // Base address
       *tc_trim_strategy = (int *)   &trim_command[4];  // + 4 bytes
double *tc_alpha         = (double *)&trim_command[8],  // + 4 bytes
       *tc_speed         = (double *)&trim_command[16], // + 8 bytes
       *tc_dele          = (double *)&trim_command[24], // + 8 bytes
       *tc_x             = (double *)&trim_command[32], // + 8 bytes
       *tc_y             = (double *)&trim_command[40], // + 8 bytes
       *tc_altitude      = (double *)&trim_command[48], // + 8 bytes
       *tc_psi           = (double *)&trim_command[56]; // + 8 bytes
                        // Last address: 63   Total size: 64 bytes

// Response message
char trim_response[80];

// Response fields
int    *tr_command_value = (int *)   &trim_response[0],  // Base address
       *tr_trim_result   = (int *)   &trim_response[4];  // + 4 bytes
double *tr_alpha         = (double *)&trim_response[8],  // + 4 bytes
       *tr_beta          = (double *)&trim_response[16], // + 8 bytes
       *tr_speed         = (double *)&trim_response[24], // + 8 bytes
       *tr_dela          = (double *)&trim_response[32], // + 8 bytes
       *tr_dele          = (double *)&trim_response[40], // + 8 bytes
       *tr_delr          = (double *)&trim_response[48], // + 8 bytes
       *tr_thrust        = (double *)&trim_response[56], // + 8 bytes
       *tr_power         = (double *)&trim_response[64], // + 8 bytes
       *tr_throttle      = (double *)&trim_response[72]; // + 8 bytes
                        // Last address: 79   Total size: 80 bytes


/* ====================  License Statement  =========================
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
   =============================================================== */
