\ ===================================================================
\  File: Commands.FTH
\
\  This file provides words that build command messages for the
\  flight simulation, send the commands then receive the replies.
\
\   -- All of these words return TRUE if communication succeeded,
\      FALSE if a network failure occured.  In the event of a failure,
\      the failure code can be accessed by calling ?SOCKERR ( -- n ).
\      The error value can be displayed by passing that value to
\      .ERRDEF ( n -- ).
\
\   : SAY-HELLO ( -- t )   Sends the HELLO command to the simulation
\      then displays the time and date from the response.
\
\   : SET-POSITION ( rX,rY,rAlt,rHeading -- )   Given X,Y and Altitude
\      in feet, and heading in degrees, sets the initial values there.
\      -- Set before trimming the aircraft. --
\
\   : TRIM-ALPHA ( r -- t )   Takes the alpha to trim the aircraft
\      in degrees, commands the simulation to trim the aircraft,
\      then sets the appropriate variables from the response.  If
\      TR-TRIM-RESULT is FALSE, there is no solution for that Alpha.
\
\   : TRIM-SPEED ( r -- t )   Takes the speed to trim the aircraft
\      in knots, commands the simulation to trim the aircraft,
\      then sets the appropriate variables from the response.
\
\   : TRIM-DELE ( r -- t )   Takes the elevator control angle to
\      trim the aircraft in degrees, commands the simulation to trim
\      the aircraft, then sets the appropriate variables from the
\      response.  If TR-TRIM-RESULT is FALSE, there is no solution
\      for that delta-e.
\
\   : .TRIM ( -- )   Displays the trim state of the aircraft.
\
\   : ADVANCE ( r -- t )   Takes the time to advance the simulation
\      in seconds, then commands the simulation to calculate that far
\      into the future.  Sets all of the variables from the response.
\
\              Copyright (C) 2014 Richard C. Wagner
\  This file is offered under the GNU General Public License.
\  Please see the license statement at the end of the file.
\ ===================================================================

\ =======================  Hello Command  ===========================

: .HR-TIME ( -- )   HELLO-RESPONSE
   DUP HR-HOURS   @ 0 <# #S #> TYPE
   DUP HR-MINUTES @ 0 <# # # [CHAR] : HOLD #> TYPE
   DUP HR-SECONDS @ 0 <# # # [CHAR] : HOLD #> TYPE
   ."   "
   DUP HR-MONTH   @ 0 <# #S #> TYPE
   DUP HR-DAY     @ 0 <# #S [CHAR] / HOLD #> TYPE
       HR-YEAR    @ 0 <# #S [CHAR] / HOLD #> TYPE ;

: SAY-HELLO ( -- t )   HELLO(CMD) HELLO-COMMAND HC-COMMAND-VALUE !
   HELLO-COMMAND  HELLO-CMD-FIELDS  SEND-COMMAND
   HELLO-RESPONSE HELLO-RESP-FIELDS RECEIVE-RESPONSE OR
   CR ." Time from simulation: " .HR-TIME ;


\ =======================  Trim Commands  ===========================

: TRIM ( -- t )   TRIM(CMD) TRIM-COMMAND TC-COMMAND-VALUE !
   TRIM-COMMAND  TRIM-CMD-FIELDS  SEND-COMMAND
   TRIM-RESPONSE TRIM-RESP-FIELDS RECEIVE-RESPONSE OR ;

: .TRIM ( -- )   TRIM-RESPONSE
   CR DUP TR-TRIM-RESULT @
      IF ." --- Success --- " ELSE ." >>> Failed <<<" THEN
   CR ." Alpha = "    DUP TR-ALPHA    DF@ RAD>DEG F.
   CR ." Beta  = "    DUP TR-BETA     DF@ RAD>DEG F.
   CR ." Speed = "    DUP TR-SPEED    DF@ 6080.0 F/ 3600.0 F* F.
   CR ." DelA  = "    DUP TR-DELA     DF@ RAD>DEG F.
   CR ." DelE  = "    DUP TR-DELE     DF@ RAD>DEG F.
   CR ." DelR  = "    DUP TR-DELR     DF@ RAD>DEG F.
   CR ." Thrust = "   DUP TR-THRUST   DF@ F.
   CR ." Throttle = "     TR-THROTTLE DF@ F. ;

\ Takes position in feet, heading in degrees.
: SET-POSITION ( rX,rY,rAlt,rPsi -- )   TRIM-COMMAND
   DUP DEG>RAD TC-Psi DF!
   DUP FNEGATE TC-Z   DF!
   DUP         TC-Y   DF!
               TC-X   DF! ;

\ Copies control deflections from TRIM-RESPONSE to ADVANCE-COMMAND.
: COPY-DELTAS ( -- )   TRIM-RESPONSE ADVANCE-COMMAND
   OVER TR-DELA     DF@      DUP AC-DELA DF!
   OVER TR-DELE     DF@      DUP AC-DELE DF!
   OVER TR-DELR     DF@      DUP AC-DELR DF!
   SWAP TR-THROTTLE DF@          AC-THROTTLE DF! ;

: SET-ALPHA ( rAlpha -- )
   BY-ALPHA  TRIM-COMMAND TC-TRIM-STRATEGY !
             TRIM-COMMAND TC-ALPHA         DF! ;

: SET-SPEED ( rSpeed -- )
   BY-SPEED  TRIM-COMMAND TC-TRIM-STRATEGY !
             TRIM-COMMAND TC-SPEED         DF! ;

: SET-DELE ( rDelE -- )
   BY-DELE   TRIM-COMMAND TC-TRIM-STRATEGY !
             TRIM-COMMAND TC-DELE          DF! ;

\ Takes Alpha in degrees.  Returns TRUE on success.
: TRIM-ALPHA ( rAlpha -- t )   SET-ALPHA TRIM COPY-DELTAS ;

\ Takes speed in kts.  Returns TRUE on success.
: TRIM-SPEED ( rSpeed -- )   SET-SPEED TRIM COPY-DELTAS ;

\ Takes DelE in degrees.  Returns TRUE on success.
: TRIM-DELE ( rDelE -- )   SET-DELE TRIM COPY-DELTAS ; 


\ =======================  Advance Command  =========================

: SET-ADVANCE ( rSec -- )
   ADVANCE(CMD) ADVANCE-COMMAND AC-COMMAND-VALUE !
                ADVANCE-COMMAND AC-ADVANCE-TIME  DF! ;

\ Returns TRUE on success.
: ADVANCE ( rSec -- t )   SET-ADVANCE
   ADVANCE-COMMAND  ADVANCE-CMD-FIELDS  SEND-COMMAND
   ADVANCE-RESPONSE ADVANCE-RESP-FIELDS RECEIVE-RESPONSE OR ;


\ ====================  License Statement  ==========================
\ This program is free software: you can redistribute it and/or modify
\ it under the terms of the GNU General Public License as published by
\ the Free Software Foundation, either version 3 of the License, or
\ (at your option) any later version.
\
\ This program is distributed in the hope that it will be useful,
\ but WITHOUT ANY WARRANTY; without even the implied warranty of
\ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
\ GNU General Public License for more details.
\
\ You should have received a copy of the GNU General Public License
\ along with this program.  If not, see <http://www.gnu.org/licenses/>.
\ ===================================================================